# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# autogenerated code
"""Module providing the job resource."""
from marshmallow.utils import missing

from ansys.hps.client.common import Object

from ..schema.job import JobSchema


class Job(Object):
    """Provides the job resource.

    Parameters
    ----------
    id : str, optional
        Unique ID to access the resource, generated internally by the server on creation.
    creation_time : datetime, optional
        Date and time that the resource was created.
    modification_time : datetime, optional
        Date and time that the resource was last modified.
    created_by : str, optional
        ID of the user who created the object.
    modified_by : str, optional
        ID of the user who last modified the object.
    name : str, optional
        Name of the job.
    eval_status : str
        Evaluation status.
    job_definition_id : str
        ID of the linked job definition. For more information, see the :class:`JobDefinition` class.
    priority : int, optional
        Priority for evaluating the job. The default is ``0``, which is the highest priority. Assigning a higher value to a job makes it a lower priority.
    values : dict[str, any], optional
        Dictionary with (name,value) pairs for all parameters defined in the linked job definition.
    fitness : float, optional
        Fitness value computed.
    fitness_term_values : dict[str, float], optional
        Dictionary with (name,value) pairs for all fitness terms computed.
    note : str, optional
        Note for the job.
    creator : str, optional
        Additional information about the creator of the job.
    executed_level : int, optional
        Execution level of the last executed task. A value of ``-1`` indicates that no task has been executed yet.
    elapsed_time : float
        Number of seconds it took the evaluators to update the job.
    host_ids : list, optional
        List of host IDs of the evaluators that updated the job.
    file_ids : list[str]
        List of IDs of all files of the job.
    """

    class Meta:
        schema = JobSchema
        rest_name = "jobs"

    def __init__(
        self,
        id=missing,
        creation_time=missing,
        modification_time=missing,
        created_by=missing,
        modified_by=missing,
        name=missing,
        eval_status=missing,
        job_definition_id=missing,
        priority=missing,
        values=missing,
        fitness=missing,
        fitness_term_values=missing,
        note=missing,
        creator=missing,
        executed_level=missing,
        elapsed_time=missing,
        host_ids=missing,
        file_ids=missing,
        **kwargs
    ):
        self.id = id
        self.creation_time = creation_time
        self.modification_time = modification_time
        self.created_by = created_by
        self.modified_by = modified_by
        self.name = name
        self.eval_status = eval_status
        self.job_definition_id = job_definition_id
        self.priority = priority
        self.values = values
        self.fitness = fitness
        self.fitness_term_values = fitness_term_values
        self.note = note
        self.creator = creator
        self.executed_level = executed_level
        self.elapsed_time = elapsed_time
        self.host_ids = host_ids
        self.file_ids = file_ids

        self.obj_type = self.__class__.__name__


JobSchema.Meta.object_class = Job



# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Module providing the Python interface to the Authorization Service API."""

from typing import List

from keycloak import KeycloakAdmin

from ..resource import User
from ..schema.user import UserSchema


class AuthApi:
    """Provides the Python interface to the Authorization Service API.

    Admin users with the Keycloak "manage-users" role can create
    users as well as modify or delete existing users. Non-admin users are only allowed
    to query the list of existing users.

    Parameters
    ----------
    client : Client
        HPS client object.

    Examples
    --------

    Get users whose first name contains ``john``.

    >>> from ansys.hps.client import Client
    >>> from ansys.hps.client.auth import AuthApi, User
    >>> cl = Client(
    ...     url="https://127.0.0.1:8443/hps", username="repuser", password="repuser"
    ... )
    >>> auth_api = AuthApi(cl)
    >>> users = auth_api.get_users(firstName="john", exact=False)

    Create a user:

    >>> new_user = User(
    ...     username="new_user",
    ...     password="dummy",
    ...     email=f"new_user@test.com",
    ...     first_name="New",
    ...     last_name="User",
    ... )
    >>> auth_api.create_user(new_user)

    """

    def __init__(self, client):
        self.client = client

    @property
    def url(self):
        """API URL."""
        return f"{self.client.url}/auth/"

    @property
    def keycloak_admin_client(self) -> KeycloakAdmin:
        """Authenticated client for the Keycloak Admin API."""
        return _admin_client(self.client)

    def get_users(self, as_objects=True, **query_params) -> List[User]:
        """Get users, filtered according to query parameters.

        Examples of query parameters are:
        - ``username``
        - ``firstName``
        - ``lastName``
        - ``exact``

        Pagination is also supported using the ``first`` and ``max`` parameters.

        For a list of supported query parameters, see the Keycloak API documentation.
        """
        return get_users(self.keycloak_admin_client, as_objects=as_objects, **query_params)

    def get_user(self, id: str) -> User:
        """Get the user representation for a given user ID."""
        return get_user(self.keycloak_admin_client, id)

    def get_user_groups(self, id: str) -> List[str]:
        """Get the groups that the user belongs to."""
        return [g["name"] for g in self.keycloak_admin_client.get_user_groups(id)]

    def get_user_realm_roles(self, id: str) -> List[str]:
        """Get the realm roles for the user."""
        return [r["name"] for r in self.keycloak_admin_client.get_realm_roles_of_user(id)]

    def user_is_admin(self, id: str) -> bool:
        """Determine if the user is a system administrator."""

        from ansys.hps.client.jms import JmsApi

        # the admin keys are configurable settings of JMS
        # they need to be queried, can't be hardcoded
        jms_api = JmsApi(self.client)
        admin_keys = jms_api.get_api_info()["settings"]["admin_keys"]

        # query user groups and roles and store in the same format
        # as admin keys
        user_keys = [f"groups.{name}" for name in self.get_user_groups(id)] + [
            f"roles.{name}" for name in self.get_user_realm_roles(id)
        ]

        # match admin and user keys
        if set(admin_keys).intersection(user_keys):
            return True

        return False

    def create_user(self, user: User, as_objects=True) -> User:
        """Create a user.

        Parameters
        ----------
        user : :class:`ansys.hps.client.auth.User`
            User object. The default is ``None``.
        as_objects : bool, optional
            The default is ``True``.
        """
        return create_user(self.keycloak_admin_client, user, as_objects=as_objects)

    def update_user(self, user: User, as_objects=True) -> User:
        """Modify an existing user.

        Parameters
        ----------
        user : :class:`ansys.hps.client.auth.User`
            User object. The default is ``None``.
        as_objects : bool, optional
            The default is  ``True``.
        """
        return update_user(self.keycloak_admin_client, user, as_objects=as_objects)

    def delete_user(self, user: User) -> None:
        """Delete an existing user.

        Parameters
        ----------
        user : :class:`ansys.hps.client.auth.User`
            User object. The default is ``None``.
        """
        return self.keycloak_admin_client.delete_user(user.id)


def _admin_client(client):
    """Set information for admin.

    Parameters
    ----------
    client : Client
        HPS client object.
    """
    custom_headers = {
        "Authorization": "Bearer " + client.access_token,
        "Content-Type": "application/json",
    }
    keycloak_admin = KeycloakAdmin(
        server_url=client.auth_api_url,
        username=None,
        password=None,
        realm_name=client.realm,
        client_id=client.client_id,
        verify=False,
        custom_headers=custom_headers,
    )
    return keycloak_admin


def get_users(admin_client: KeycloakAdmin, as_objects=True, **query_params):
    """Get users as admin.

    Parameters
    ----------
    admin_client : KeycloakAdmin
        Keycloak admin user.
    """
    users = admin_client.get_users(query=query_params)

    if not as_objects:
        return users

    schema = UserSchema(many=True)
    return schema.load(users)


def get_user(admin_client: KeycloakAdmin, id: str, as_objects=True):
    """Get user using ID.

    Parameters
    ----------
    admin_client: KeycloakAdmin
        Keycloak admin user.
    id : str
        User ID.
    """
    user = admin_client.get_user(user_id=id)

    if not as_objects:
        return user

    schema = UserSchema(many=False)
    return schema.load(user)


def create_user(admin_client: KeycloakAdmin, user: User, as_objects=True):
    """Create user.

    Parameters
    ----------
    admin_client : KeycloakAdmin
        Keycloak admin user.
    user : User
        HPS user object.
    """
    schema = UserSchema(many=False)
    data = schema.dump(user)

    pwd = data.pop("password", None)
    if pwd is not None:
        data["credentials"] = [
            {
                "type": "password",
                "value": pwd,
            }
        ]
    data["enabled"] = True

    uid = admin_client.create_user(data)
    return get_user(admin_client, uid, as_objects)


def update_user(admin_client: KeycloakAdmin, user: User, as_objects=True):
    """Update user.

    Parameters
    ----------
    admin_client : KeycloakAdmin
        Keycloak admin user.
    user : User
        HPS user object.
    """
    schema = UserSchema(many=False)
    data = schema.dump(user)

    pwd = data.pop("password", None)
    if pwd is not None:
        data["credentials"] = [
            {
                "type": "password",
                "value": pwd,
            }
        ]

    data = admin_client.update_user(user.id, data)

    if not as_objects:
        return data

    user = schema.load(data)
    return user



# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# autogenerated code
"""Module providing the project resource."""
from marshmallow.utils import missing

from ansys.hps.client.common import Object

from ..schema.project import ProjectSchema


class Project(Object):
    """Provides the project resource.

    Parameters
    ----------
    id : str
        Unique ID to access the project, assigned server side on creation.
    name : str
        Name of the project.
    active : bool
        Defines whether the project is active for evaluation.
    priority : int
        Priority for picking the project for evaluation.
    creation_time : datetime, optional
        Date and time that the project was created.
    modification_time : datetime, optional
        Date and time that the project was last modified.
    statistics : dict, optional
        Dictionary containing various project statistics.
    """

    class Meta:
        schema = ProjectSchema
        rest_name = "projects"

    def __init__(
        self,
        id=missing,
        name=missing,
        active=missing,
        priority=missing,
        creation_time=missing,
        modification_time=missing,
        statistics=missing,
        **kwargs
    ):
        self.id = id
        self.name = name
        self.active = active
        self.priority = priority
        self.creation_time = creation_time
        self.modification_time = modification_time
        self.statistics = statistics

        self.obj_type = self.__class__.__name__


ProjectSchema.Meta.object_class = Project



# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# autogenerated code
"""Module providing the operation resource."""
from marshmallow.utils import missing

from ansys.hps.client.common import Object

from ..schema.operation import OperationSchema


class Operation(Object):
    """Provides the operation resource.

    Parameters
    ----------
    id : str, optional
        Unique ID to access the resource, generated internally by the server on creation.
    name : str, optional
    target : list, optional
    finished : bool, optional
    succeeded : bool, optional
    progress : float, optional
    status : str, optional
    result : dict, optional
    messages : list, optional
    start_time : datetime, optional
    end_time : datetime, optional
    """

    class Meta:
        schema = OperationSchema
        rest_name = "operations"

    def __init__(
        self,
        id=missing,
        name=missing,
        target=missing,
        finished=missing,
        succeeded=missing,
        progress=missing,
        status=missing,
        result=missing,
        messages=missing,
        start_time=missing,
        end_time=missing,
        **kwargs
    ):
        self.id = id
        self.name = name
        self.target = target
        self.finished = finished
        self.succeeded = succeeded
        self.progress = progress
        self.status = status
        self.result = result
        self.messages = messages
        self.start_time = start_time
        self.end_time = end_time

        self.obj_type = self.__class__.__name__


OperationSchema.Meta.object_class = Operation



# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Module providing job definition schema."""
import logging

from marshmallow import fields

from ansys.hps.client.common import ObjectSchemaWithModificationInfo

from .fitness_definition import FitnessDefinitionSchema
from .object_reference import IdReferenceList

log = logging.getLogger(__name__)


class JobDefinitionSchema(ObjectSchemaWithModificationInfo):
    class Meta(ObjectSchemaWithModificationInfo.Meta):
        pass

    name = fields.String(allow_none=True, metadata={"description": "Name of the job definition."})
    active = fields.Boolean(metadata={"description": "Whether this job definition is active."})
    client_hash = fields.String(allow_none=True)

    parameter_definition_ids = IdReferenceList(
        referenced_class="ParameterDefinition",
        attribute="parameter_definition_ids",
        metadata={"description": "List of parameter definition IDs."},
    )
    parameter_mapping_ids = IdReferenceList(
        referenced_class="ParameterMapping",
        attribute="parameter_mapping_ids",
        metadata={"description": "List of parameter mapping IDs."},
    )
    task_definition_ids = IdReferenceList(
        referenced_class="TaskDefinition",
        attribute="task_definition_ids",
        metadata={"description": "List of task definition IDs."},
    )

    fitness_definition = fields.Nested(
        FitnessDefinitionSchema,
        allow_none=True,
        metadata={"description": "A :class:`FitnessDefinition` object."},
    )



# Copyright (C) 2022 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# autogenerated code
"""Module providing the task resource."""
from marshmallow.utils import missing

from ansys.hps.client.common import Object

from ..schema.task import TaskSchema


class Task(Object):
    """Provides the task resource.

    Parameters
    ----------
    id : str, optional
        Unique ID to access the resource, generated internally by the server on creation.
    creation_time : datetime, optional
        Date and time that the resource was created.
    modification_time : datetime, optional
        Date and time that the resource was last modified.
    created_by : str, optional
        ID of the user who created the object.
    modified_by : str, optional
        ID of the user who last modified the object.
    pending_time : datetime, optional
        Date and time that the task was set to pending.
    prolog_time : datetime, optional
        Date and time that the task was set to prolog.
    running_time : datetime, optional
        Date and time that the task was set to running.
    finished_time : datetime, optional
        Date and time that the task was completed.
    eval_status : str, optional
        Evaluation status.
    trial_number : int, optional
        Which attempt to execute the process step this task represents.
    elapsed_time : float, optional
        Number of seconds it took the evaluator to execute the task.
    task_definition_id : str
        ID of the :class:`TaskDefinition` instance that the task is linked to.
    task_definition_snapshot : TaskDefinition, optional
        Snapshot of the  :class:`TaskDefinition` instance that was created when the task status changed to prolog, before evaluation.
    executed_command : str, optional
    job_id : str
        ID of the :class:`Job` instance that the task is linked to.
    host_id : str, optional
        UUID of the :class:`Evaluator` instance that updated the task.
    input_file_ids : list[str]
        List of IDs of input files of the task.
    output_file_ids : list[str]
        List of IDs of output files of the task.
    monitored_file_ids : list[str]
        List of IDs of monitored files of the task.
    inherited_file_ids : list[str]
        List of IDs of inherited files of the task.
    owned_file_ids : list[str]
        List of IDs of owned files of the task.
    license_context_id : str, optional
        ID of the license context in use.
    custom_data : dict, optional
        Dictionary type field for storing custom data.
    working_directory : str, optional
        Working directory of the task.
    """

    class Meta:
        schema = TaskSchema
        rest_name = "tasks"

    def __init__(
        self,
        id=missing,
        creation_time=missing,
        modification_time=missing,
        created_by=missing,
        modified_by=missing,
        pending_time=missing,
        prolog_time=missing,
        running_time=missing,
        finished_time=missing,
        eval_status=missing,
        trial_number=missing,
        elapsed_time=missing,
        task_definition_id=missing,
        task_definition_snapshot=missing,
        executed_command=missing,
        job_id=missing,
        host_id=missing,
        input_file_ids=missing,
        output_file_ids=missing,
        monitored_file_ids=missing,
        inherited_file_ids=missing,
        owned_file_ids=missing,
        license_context_id=missing,
        custom_data=missing,
        working_directory=missing,
        **kwargs
    ):
        self.id = id
        self.creation_time = creation_time
        self.modification_time = modification_time
        self.created_by = created_by
        self.modified_by = modified_by
        self.pending_time = pending_time
        self.prolog_time = prolog_time
        self.running_time = running_time
        self.finished_time = finished_time
        self.eval_status = eval_status
        self.trial_number = trial_number
        self.elapsed_time = elapsed_time
        self.task_definition_id = task_definition_id
        self.task_definition_snapshot = task_definition_snapshot
        self.executed_command = executed_command
        self.job_id = job_id
        self.host_id = host_id
        self.input_file_ids = input_file_ids
        self.output_file_ids = output_file_ids
        self.monitored_file_ids = monitored_file_ids
        self.inherited_file_ids = inherited_file_ids
        self.owned_file_ids = owned_file_ids
        self.license_context_id = license_context_id
        self.custom_data = custom_data
        self.working_directory = working_directory

        self.obj_type = self.__class__.__name__


TaskSchema.Meta.object_class = Task
